@@ -49,11 +49,90 @@ if TYPE_CHECKING:
     from ..start import Dominus
 
 
+# ========================================
+# Gateway-routed API helper
+# ========================================
+
+class GatewayMixin:
+    """
+    Mixin that provides gateway-routed API calls.
+
+    All AI namespace methods route through the gateway's /svc/* endpoints
+    which proxy to agent-runtime's /api/* endpoints.
+    """
+    _client: "Dominus"
+
+    async def _api(
+        self,
+        endpoint: str,
+        method: str = "POST",
+        body: Optional[Dict[str, Any]] = None
+    ) -> Dict[str, Any]:
+        """Make gateway-routed API request."""
+        return await self._client._request(
+            endpoint=endpoint,
+            method=method,
+            body=body,
+            use_gateway=True
+        )
+
+    async def _api_stream(
+        self,
+        endpoint: str,
+        body: Optional[Dict[str, Any]] = None,
+        on_chunk: Optional[Any] = None,
+        timeout: float = 300.0
+    ):
+        """Make gateway-routed streaming request."""
+        async for chunk in self._client._stream_request(
+            endpoint=endpoint,
+            body=body,
+            on_chunk=on_chunk,
+            timeout=timeout,
+            use_gateway=True
+        ):
+            yield chunk
+
+    async def _api_upload(
+        self,
+        endpoint: str,
+        file_bytes: bytes,
+        filename: str,
+        content_type: str = "application/octet-stream",
+        additional_fields: Optional[Dict[str, str]] = None,
+        timeout: float = 60.0
+    ) -> Dict[str, Any]:
+        """Make gateway-routed binary upload."""
+        return await self._client._binary_upload(
+            endpoint=endpoint,
+            file_bytes=file_bytes,
+            filename=filename,
+            content_type=content_type,
+            additional_fields=additional_fields,
+            timeout=timeout,
+            use_gateway=True
+        )
+
+    async def _api_download(
+        self,
+        endpoint: str,
+        body: Optional[Dict[str, Any]] = None,
+        timeout: float = 60.0
+    ) -> bytes:
+        """Make gateway-routed binary download."""
+        return await self._client._binary_download(
+            endpoint=endpoint,
+            body=body,
+            timeout=timeout,
+            use_gateway=True
+        )
+
+
 # ========================================
 # RAG Sub-namespace
 # ========================================
 
-class RagSubNamespace:
+class RagSubNamespace(GatewayMixin):
     """
     RAG corpus management sub-namespace.
 
@@ -65,7 +144,7 @@ class RagSubNamespace:
 
     async def list(self) -> List[Dict[str, Any]]:
         """List all corpora."""
-        result = await self._client._request(
+        result = await self._api(
             endpoint="/api/rag",
             method="GET"
         )
@@ -91,21 +170,21 @@ class RagSubNamespace:
         if embedding_model:
             body["embedding_model"] = embedding_model
 
-        return await self._client._request(
+        return await self._api(
             endpoint=f"/api/rag/{slug}/ensure",
             body=body if body else {}
         )
 
     async def stats(self, slug: str) -> Dict[str, Any]:
         """Get corpus statistics."""
-        return await self._client._request(
+        return await self._api(
             endpoint=f"/api/rag/{slug}/stats",
             method="GET"
         )
 
     async def drop(self, slug: str) -> Dict[str, Any]:
         """Drop/delete a corpus."""
-        return await self._client._request(
+        return await self._api(
             endpoint=f"/api/rag/{slug}",
             method="DELETE"
         )
@@ -122,7 +201,7 @@ class RagSubNamespace:
         if category:
             body["category"] = category
 
-        return await self._client._request(
+        return await self._api(
             endpoint=f"/api/rag/{slug}/entries",
             body=body
         )
@@ -151,7 +230,7 @@ class RagSubNamespace:
         if metadata:
             body["metadata"] = metadata
 
-        return await self._client._request(
+        return await self._api(
             endpoint=f"/api/rag/{slug}/{identifier}",
             method="PUT",
             body=body
@@ -159,14 +238,14 @@ class RagSubNamespace:
 
     async def get(self, slug: str, identifier: str) -> Dict[str, Any]:
         """Get a specific entry."""
-        return await self._client._request(
+        return await self._api(
             endpoint=f"/api/rag/{slug}/{identifier}",
             method="GET"
         )
 
     async def delete(self, slug: str, identifier: str) -> Dict[str, Any]:
         """Delete an entry."""
-        return await self._client._request(
+        return await self._api(
             endpoint=f"/api/rag/{slug}/{identifier}",
             method="DELETE"
         )
@@ -183,7 +262,7 @@ class RagSubNamespace:
             slug: Corpus identifier
             entries: List of {"identifier", "content", "category", "metadata"} dicts
         """
-        return await self._client._request(
+        return await self._api(
             endpoint=f"/api/rag/{slug}/bulk",
             body={"entries": entries}
         )
@@ -222,7 +301,7 @@ class RagSubNamespace:
         if filters:
             body["filters"] = filters
 
-        result = await self._client._request(
+        result = await self._api(
             endpoint=f"/api/rag/{slug}/search",
             body=body
         )
@@ -251,7 +330,7 @@ class RagSubNamespace:
         if rerank_model:
             body["rerank_model"] = rerank_model
 
-        result = await self._client._request(
+        result = await self._api(
             endpoint=f"/api/rag/{slug}/search/rerank",
             body=body
         )
@@ -277,7 +356,7 @@ class RagSubNamespace:
             chunk_size: Chunk size for splitting (default: 1000 chars)
             chunk_overlap: Overlap between chunks (default: 200 chars)
         """
-        return await self._client._binary_upload(
+        return await self._api_upload(
             endpoint=f"/api/rag/{slug}/ingest",
             file_bytes=content,
             filename=filename,
@@ -290,7 +369,7 @@ class RagSubNamespace:
 
     async def delete_document(self, slug: str, document_id: str) -> Dict[str, Any]:
         """Delete a document and all its chunks."""
-        return await self._client._request(
+        return await self._api(
             endpoint=f"/api/rag/{slug}/document/{document_id}",
             method="DELETE"
         )
@@ -300,7 +379,7 @@ class RagSubNamespace:
 # Artifacts Sub-namespace
 # ========================================
 
-class ArtifactsSubNamespace:
+class ArtifactsSubNamespace(GatewayMixin):
     """
     Artifacts sub-namespace for conversation artifact management.
     """
@@ -318,7 +397,7 @@ class ArtifactsSubNamespace:
         if conversation_id:
             body["conversation_id"] = conversation_id
 
-        return await self._client._request(
+        return await self._api(
             endpoint=f"/api/agent/artifacts/{artifact_id}",
             body=body
         )
@@ -350,7 +429,7 @@ class ArtifactsSubNamespace:
         if metadata:
             body["metadata"] = metadata
 
-        return await self._client._request(
+        return await self._api(
             endpoint="/api/agent/artifacts",
             body=body
         )
@@ -368,7 +447,7 @@ class ArtifactsSubNamespace:
             "offset": offset
         }
 
-        result = await self._client._request(
+        result = await self._api(
             endpoint="/api/agent/artifacts",
             method="GET",
             body=body
@@ -385,7 +464,7 @@ class ArtifactsSubNamespace:
         if conversation_id:
             body["conversation_id"] = conversation_id
 
-        return await self._client._request(
+        return await self._api(
             endpoint=f"/api/agent/artifacts/{artifact_id}",
             method="DELETE",
             body=body if body else None
@@ -396,7 +475,7 @@ class ArtifactsSubNamespace:
 # Results Sub-namespace
 # ========================================
 
-class ResultsSubNamespace:
+class ResultsSubNamespace(GatewayMixin):
     """
     Async results sub-namespace for polling async operation results.
     """
@@ -415,7 +494,7 @@ class ResultsSubNamespace:
             Dict with "status" (pending|running|completed|failed),
             "result" (if completed), "error" (if failed)
         """
-        return await self._client._request(
+        return await self._api(
             endpoint=f"/api/results/{result_key}",
             method="GET"
         )
@@ -460,7 +539,7 @@ class ResultsSubNamespace:
 # Workflow Sub-namespace
 # ========================================
 
-class WorkflowSubNamespace:
+class WorkflowSubNamespace(GatewayMixin):
     """
     Multi-agent workflow orchestration sub-namespace.
     """
@@ -499,7 +578,7 @@ class WorkflowSubNamespace:
         if webhook_url:
             body["webhook_url"] = webhook_url
 
-        return await self._client._request(
+        return await self._api(
             endpoint="/api/orchestration/execute",
             body=body
         )
@@ -517,7 +596,7 @@ class WorkflowSubNamespace:
         Returns:
             Dict with "valid", "errors" (if invalid)
         """
-        return await self._client._request(
+        return await self._api(
             endpoint="/api/orchestration/validate",
             body={"definition": workflow_definition}
         )
@@ -529,7 +608,7 @@ class WorkflowSubNamespace:
         offset: int = 0
     ) -> List[Dict[str, Any]]:
         """Get messages from an execution."""
-        result = await self._client._request(
+        result = await self._api(
             endpoint=f"/api/orchestration/messages/{execution_id}",
             body={"limit": limit, "offset": offset}
         )
@@ -545,7 +624,7 @@ class WorkflowSubNamespace:
         if from_timestamp:
             body["from"] = from_timestamp
 
-        result = await self._client._request(
+        result = await self._api(
             endpoint=f"/api/orchestration/events/{execution_id}",
             body=body if body else None
         )
@@ -553,14 +632,14 @@ class WorkflowSubNamespace:
 
     async def status(self, execution_id: str) -> Dict[str, Any]:
         """Get execution status."""
-        return await self._client._request(
+        return await self._api(
             endpoint=f"/api/orchestration/status/{execution_id}",
             method="GET"
         )
 
     async def output(self, execution_id: str) -> Dict[str, Any]:
         """Get final execution output."""
-        return await self._client._request(
+        return await self._api(
             endpoint=f"/api/orchestration/output/{execution_id}",
             method="GET"
         )
@@ -570,7 +649,7 @@ class WorkflowSubNamespace:
 # Main AI Namespace
 # ========================================
 
-class AiNamespace:
+class AiNamespace(GatewayMixin):
     """
     Unified AI namespace for agent-runtime operations.
 
@@ -656,7 +735,7 @@ class AiNamespace:
         if tool_endpoint:
             body["tool_endpoint"] = tool_endpoint
 
-        return await self._client._request(
+        return await self._api(
             endpoint="/api/agent/run",
             body=body
         )
@@ -722,7 +801,7 @@ class AiNamespace:
         if tool_endpoint:
             body["tool_endpoint"] = tool_endpoint
 
-        async for chunk in self._client._stream_request(
+        async for chunk in self._api_stream(
             endpoint="/api/agent/stream",
             body=body,
             on_chunk=on_chunk,
@@ -796,7 +875,7 @@ class AiNamespace:
         if webhook_url:
             body["webhook_url"] = webhook_url
 
-        return await self._client._request(
+        return await self._api(
             endpoint="/api/agent/run-async",
             body=body
         )
@@ -820,7 +899,7 @@ class AiNamespace:
         Returns:
             List of history messages
         """
-        result = await self._client._request(
+        result = await self._api(
             endpoint=f"/api/agent/history/{conversation_id}",
             body={"limit": limit}
         )
@@ -832,58 +911,79 @@ class AiNamespace:
 
     async def complete(
         self,
-        messages: List[Dict[str, Any]],
+        messages: Optional[List[Dict[str, Any]]] = None,
         provider: str = "claude",
         model: str = "claude-sonnet-4-5",
         conversation_id: Optional[str] = None,
         temperature: float = 0.7,
         max_tokens: Optional[int] = None,
         system_prompt: Optional[str] = None,
+        user_prompt: Optional[str] = None,
         timeout: float = 120.0
     ) -> Dict[str, Any]:
         """
         Blocking LLM completion.
 
         Args:
-            messages: List of message dicts with "role" and "content"
+            messages: List of message dicts with "role" and "content" (optional)
             provider: LLM provider ("claude" or "openai")
             model: Model identifier
             conversation_id: Optional conversation ID
             temperature: Temperature (0-2, default: 0.7)
             max_tokens: Maximum tokens to generate
-            system_prompt: Optional system prompt
+            system_prompt: System prompt (required if messages not provided)
+            user_prompt: User prompt (required if messages not provided)
             timeout: Request timeout in seconds
 
         Returns:
             Dict with "content", "model", "usage", etc.
+
+        Note:
+            Either provide `messages` OR both `system_prompt` and `user_prompt`.
+            If messages provided, the last user message becomes user_prompt,
+            and system messages become system_prompt.
         """
+        # Convert messages to system_prompt/user_prompt if needed
+        if messages and not user_prompt:
+            for msg in messages:
+                if msg.get("role") == "system":
+                    system_prompt = msg.get("content", system_prompt)
+                elif msg.get("role") == "user":
+                    user_prompt = msg.get("content")
+
+        if not system_prompt:
+            system_prompt = "You are a helpful assistant."
+
+        if not user_prompt:
+            raise ValueError("Either 'messages' with a user message or 'user_prompt' is required")
+
         body: Dict[str, Any] = {
-            "messages": messages,
             "provider": provider,
             "model": model,
+            "system_prompt": system_prompt,
+            "user_prompt": user_prompt,
             "temperature": temperature
         }
         if conversation_id:
             body["conversation_id"] = conversation_id
         if max_tokens:
             body["max_tokens"] = max_tokens
-        if system_prompt:
-            body["system_prompt"] = system_prompt
 
-        return await self._client._request(
+        return await self._api(
             endpoint="/api/llm/complete",
             body=body
         )
 
     async def complete_stream(
         self,
-        messages: List[Dict[str, Any]],
+        messages: Optional[List[Dict[str, Any]]] = None,
         provider: str = "claude",
         model: str = "claude-sonnet-4-5",
         conversation_id: Optional[str] = None,
         temperature: float = 0.7,
         max_tokens: Optional[int] = None,
         system_prompt: Optional[str] = None,
+        user_prompt: Optional[str] = None,
         on_chunk: Optional[Callable[[Dict[str, Any]], None]] = None,
         timeout: float = 120.0
     ) -> AsyncGenerator[Dict[str, Any], None]:
@@ -891,33 +991,47 @@ class AiNamespace:
         Streaming LLM completion.
 
         Args:
-            messages: List of message dicts
+            messages: List of message dicts (optional)
             provider: LLM provider
             model: Model identifier
             conversation_id: Optional conversation ID
             temperature: Temperature (0-2)
             max_tokens: Maximum tokens
-            system_prompt: Optional system prompt
+            system_prompt: System prompt (required if messages not provided)
+            user_prompt: User prompt (required if messages not provided)
             on_chunk: Optional callback for each chunk
             timeout: Request timeout
 
         Yields:
             Streaming chunks with "content" delta
         """
+        # Convert messages to system_prompt/user_prompt if needed
+        if messages and not user_prompt:
+            for msg in messages:
+                if msg.get("role") == "system":
+                    system_prompt = msg.get("content", system_prompt)
+                elif msg.get("role") == "user":
+                    user_prompt = msg.get("content")
+
+        if not system_prompt:
+            system_prompt = "You are a helpful assistant."
+
+        if not user_prompt:
+            raise ValueError("Either 'messages' with a user message or 'user_prompt' is required")
+
         body: Dict[str, Any] = {
-            "messages": messages,
             "provider": provider,
             "model": model,
+            "system_prompt": system_prompt,
+            "user_prompt": user_prompt,
             "temperature": temperature
         }
         if conversation_id:
             body["conversation_id"] = conversation_id
         if max_tokens:
             body["max_tokens"] = max_tokens
-        if system_prompt:
-            body["system_prompt"] = system_prompt
 
-        async for chunk in self._client._stream_request(
+        async for chunk in self._api_stream(
             endpoint="/api/llm/stream",
             body=body,
             on_chunk=on_chunk,
@@ -970,7 +1084,7 @@ class AiNamespace:
         if webhook_url:
             body["webhook_url"] = webhook_url
 
-        return await self._client._request(
+        return await self._api(
             endpoint="/api/llm/complete-async",
             body=body
         )
@@ -996,7 +1110,7 @@ class AiNamespace:
         Returns:
             Dict with "text", "confidence", "language", "duration"
         """
-        return await self._client._binary_upload(
+        return await self._api_upload(
             endpoint="/api/agent/stt",
             file_bytes=audio,
             filename=f"audio.{format}",
@@ -1023,7 +1137,7 @@ class AiNamespace:
         Returns:
             Raw audio bytes (NOT base64 encoded)
         """
-        return await self._client._binary_download(
+        return await self._api_download(
             endpoint="/api/agent/tts",
             body={
                 "text": text,
@@ -1070,7 +1184,7 @@ class AiNamespace:
         if context:
             body["context"] = context
 
-        return await self._client._request(
+        return await self._api(
             endpoint="/api/session/setup",
             body=body
         )
